__all__ = ["parser"]

from icevision.all import *
from icevision.core.record_defaults import KeypointsRecord


def parser(annotations_path: Union[str, Path], img_dir_path: Union[str, Path]):
    return OCHumanParser(
        annotations_filepath=Path(annotations_path), img_dir=Path(img_dir_path)
    )


class OCHConnectionsColor:
    head = (220, 30, 200)
    torso = (100, 240, 100)
    right_arm = (170, 170, 100)
    left_arm = (120, 120, 230)
    left_leg = (230, 120, 130)
    right_leg = (230, 180, 190)


class OCHKeypointsMetadata(KeypointsMetadata):
    labels = (
        "right_shoulder",
        "right_elbow",
        "right_wrist",
        "left_shoulder",
        "left_elbow",
        "left_wrist",
        "right_hip",
        "right_knee",
        "right_ankle",
        "left_hip",
        "left_knee",
        "left_ankle",
        "head",
        "neck",
        "right_ear",
        "left_ear",
        "nose",
        "right_eye",
        "left_eye",
    )

    # connections = (
    #     KeypointConnection(15, 18, OCHConnectionsColor.head),
    #     KeypointConnection(12, 16, OCHConnectionsColor.head),
    #     KeypointConnection(3, 4, OCHConnectionsColor.left_arm),
    #     KeypointConnection(18, 16, OCHConnectionsColor.head),
    #     KeypointConnection(16, 13, OCHConnectionsColor.head),
    #     KeypointConnection(4, 5, OCHConnectionsColor.left_arm),
    #     KeypointConnection(16, 17, OCHConnectionsColor.head),
    #     KeypointConnection(13, 3, OCHConnectionsColor.torso),
    #     KeypointConnection(0, 1, OCHConnectionsColor.right_arm),
    #     KeypointConnection(17, 14, OCHConnectionsColor.head),
    #     KeypointConnection(13, 0, OCHConnectionsColor.torso),
    #     KeypointConnection(1, 2, OCHConnectionsColor.right_arm),
    #     KeypointConnection(3, 9, OCHConnectionsColor.torso),
    #     KeypointConnection(0, 6, OCHConnectionsColor.torso),
    #     KeypointConnection(9, 6, OCHConnectionsColor.torso),
    #     KeypointConnection(9, 10, OCHConnectionsColor.left_leg),
    #     KeypointConnection(6, 7, OCHConnectionsColor.right_leg),
    #     KeypointConnection(10, 11, OCHConnectionsColor.left_leg),
    #     KeypointConnection(7, 8, OCHConnectionsColor.right_leg),
    #     KeypointConnection(15, 3, OCHConnectionsColor.head),
    #     KeypointConnection(14, 0, OCHConnectionsColor.head),
    # )


class OCHumanParser(Parser):
    def __init__(self, annotations_filepath, img_dir):
        super().__init__(template_record=self.template_record())
        self.annotations_dict = json.loads(Path(annotations_filepath).read_bytes())
        self.img_dir = Path(img_dir)
        self.class_map = ClassMap(["person"])

    def __iter__(self):
        yield from self.annotations_dict["images"]

    def __len__(self):
        return len(self.annotations_dict["images"])

    def template_record(self):
        return KeypointsRecord()

    def record_id(self, o):
        return int(o["image_id"])

    def parse_fields(self, o, record: BaseRecord, is_new: bool) -> None:
        if is_new:
            record.set_filepath(self.filepath(o))
            record.set_img_size(self.image_width_height(o))
            record.detection.set_class_map(self.class_map)

        record.detection.add_labels(self.labels(o))
        record.detection.add_bboxes(self.bboxes(o))
        record.detection.add_keypoints(self.keypoints(o))

    def filepath(self, o):
        return self.img_dir / o["file_name"]

    def keypoints(self, o):
        return [
            KeyPoints.from_xyv(kps["keypoints"], OCHKeypointsMetadata)
            for kps in o["annotations"]
            if kps["keypoints"] is not None
        ]

    def image_width_height(self, o) -> Tuple[int, int]:
        return get_img_size(self.filepath(o))

    def labels(self, o) -> List[Hashable]:
        return ["person" for ann in o["annotations"] if ann["keypoints"] is not None]

    def bboxes(self, o) -> List[BBox]:
        return [
            BBox.from_xyxy(*ann["bbox"])
            for ann in o["annotations"]
            if ann["keypoints"] is not None
        ]



__all__ = ["NUM_CLASSES", "load_data"]

from icevision.imports import *
from icevision import *

NUM_CLASSES = 1 + 1


def load_data(force_download=False):
    url = "https://www.cis.upenn.edu/~jshi/ped_html/PennFudanPed.zip"
    # setup file names
    save_dir = get_data_dir() / "PennFudanPed"
    save_dir.mkdir(exist_ok=True)
    zip_file = save_dir / "pennfundan.zip"
    # download and extract data
    if not zip_file.exists() or force_download:
        download_url(url=url, save_path=zip_file)
        # extract file
        with zipfile.ZipFile(zip_file, "r") as f:
            f.extractall(save_dir)
        # move all extracted files a directory up so they end up at save_dir
        files_dir = save_dir / "PennFudanPed"
        for file in files_dir.ls():
            shutil.move(str(file), save_dir)
        shutil.rmtree(files_dir)

    return save_dir



__all__ = ["faster_rcnn_resnet50_fpn"]

from icevision.all import *
from icedata.utils import load_model_weights_from_url
from icedata.datasets.pets.data import NUM_CLASSES


def _load_efficientdet(name, weights_url):
    model = efficientdet.model(name, num_classes=NUM_CLASSES, img_size=512)
    load_model_weights_from_url(model, weights_url)
    return model


def _load_faster_rcnn(backbone, weights_url):
    model = models.torchvision.faster_rcnn.model(
        num_classes=NUM_CLASSES, backbone=backbone
    )
    load_model_weights_from_url(model=model, url=weights_url)
    return model


# def tf_efficientdet_lite0():
#     # weights_url = "https://github.com/airctic/icedata/releases/download/m1/pets_tf_efficientdet_lite0.zip"
#     weights_url = "https://github.com/airctic/model_zoo/releases/download/m2/pets_tf_efficientdet_lite0.zip"
#     return _load_efficientdet("tf_efficientdet_lite0", weights_url)


def faster_rcnn_resnet50_fpn():
    weights_url = "https://github.com/airctic/model_zoo/releases/download/m3/pets_faster_resnetfpn50.zip"
    return _load_faster_rcnn(None, weights_url)



import pytest
import icedata
import torch.nn as nn


@pytest.mark.parametrize("model_name", icedata.fridge.trained_models.__all__)
def test_trained_models(model_name):
    model_fn = getattr(icedata.fridge.trained_models, model_name)
    model = model_fn()
    assert isinstance(model, nn.Module)



import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.biwi.parser(data_dir)

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 6
    record = records[3]

    assert record.filepath.name == "3.jpg"
    assert record.record_id == 3
    assert record.detection.labels == ["nose"]
    assert record.height == 120
    assert record.width == 160

    expected = [BBox.from_xywh(0, 0, record.width, record.height)]
    assert record.detection.bboxes == expected

    expected = KeyPoints.from_xyv([78, 63, 1], ["nose"])
    assert [
        (int(round(x)), int(round(y)), 1)
        for x, y, v in record.detection.keypoints[0].xyv
    ] == expected.xyv



__all__ = ["mask_rcnn_resnet50_fpn"]

from icevision.all import *
from icedata.utils import load_model_weights_from_url
from icedata.datasets.birds.data import NUM_CLASSES


def mask_rcnn_resnet50_fpn() -> nn.Module:
    weights_url = "https://github.com/airctic/model_zoo/releases/download/m4/birds_maskrcnn_resnet50fpn.zip"
    model = mask_rcnn.model(num_classes=NUM_CLASSES)
    load_model_weights_from_url(model=model, url=weights_url)
    return model



__all__ = ["parser"]

from icevision.imports import *
from icevision import *
from icevision.parsers.parser import *


def parser(data_dir) -> parsers.ParserInterface:
    return PennFundanParser(data_dir=data_dir)


class PennFundanParser(Parser):
    def __init__(
        self,
        data_dir: Union[str, Path],
        class_map: Optional[ClassMap] = None,
        idmap: Optional[IDMap] = None,
    ):
        super().__init__(template_record=self.template_record(), idmap=idmap)
        self.class_map = class_map or ClassMap().unlock()
        self.data_dir = data_dir
        self.filenames = get_files(data_dir / "Annotation", extensions=".txt")

    def __iter__(self):
        yield from self.filenames

    def __len__(self):
        return len(self.filenames)

    def template_record(self) -> BaseRecord:
        return BaseRecord(
            (
                FilepathRecordComponent(),
                InstancesLabelsRecordComponent(),
                BBoxesRecordComponent(),
                InstanceMasksRecordComponent(),
            )
        )

    def prepare(self, o):
        self._record_id = getattr(self, "_record_id", 0) + 1
        self.lines = L(o.read_text().split("\n"))
        self._bboxes = []

        for line in self.lines:
            if line.startswith("Image filename"):
                filename = re.findall(r'"(.*)"', line)[0]
                self._filepath = self.data_dir.parent / filename

            elif line.startswith("Image size (X x Y x C)"):
                size_str = re.search(r"\d{3,4}\sx\s\d{3,4}\sx\s3", line).group()
                self._size = [int(o) for o in size_str.split("x")]

            elif line.startswith("Objects with ground truth"):
                # number of objects is the first number that shows in the line
                self._num_objects = int(re.findall("\d+", line)[0])

            elif line.startswith("Pixel mask for object"):
                mask_filename = re.findall(r'"(.+)"', line.split(":")[-1])[0]
                self._mask_filepath = self.data_dir.parent / mask_filename

            if line.startswith("Bounding box"):
                # find bbox coordinates in line and covert to a list
                point_pairs_str = re.findall(r"(\d+,\s\d+)", line)
                points = []
                for pairs in point_pairs_str:
                    for point in pairs.split(","):
                        points.append(int(point))

                bbox = BBox.from_xyxy(*points)
                self._bboxes.append(bbox)

    def record_id(self, o) -> int:
        return self._record_id

    def filepath(self, o) -> Union[str, Path]:
        return self._filepath

    def image_width_height(self, o) -> Tuple[int, int]:
        return self._size[:2]

    def labels(self, o) -> List[Hashable]:
        return ["person"] * self._num_objects

    def iscrowds(self, o) -> List[bool]:
        return [False] * self._num_objects

    def masks(self, o) -> List[Mask]:
        return [MaskFile(self._mask_filepath)]

    def bboxes(self, o) -> List[BBox]:
        return self._bboxes

    def parse_fields(self, o, record, is_new):
        record.set_filepath(self.filepath(o))
        record.set_img_size(self.image_width_height(o))

        record.detection.set_class_map(self.class_map)
        record.detection.add_labels(self.labels(o))
        record.detection.add_bboxes(self.bboxes(o))
        record.detection.add_masks(self.masks(o))



import icedata


def test_class_map():
    num_classes = icedata.pets.NUM_CLASSES
    class_map = icedata.pets.class_map()

    assert num_classes == len(class_map) == 38
    assert class_map.get_by_id(0) == "background"
    assert class_map.get_by_id(-1) == "yorkshire_terrier"



from icedata.datasets import birds
from icedata.datasets import coco
from icedata.datasets import fridge
from icedata.datasets import pennfudan
from icedata.datasets import pets
from icedata.datasets import voc
from icedata.datasets import biwi
from icedata.datasets import ochuman



import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.pets.parser(data_dir, mask=True)

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 5
    record = records[0]

    assert record.filepath.name == "Abyssinian_119.jpg"
    assert record.record_id == "Abyssinian_119"
    assert record.detection.labels == ["Abyssinian"]
    assert record.height == 297
    assert record.width == 300

    assert record.detection.bboxes == [BBox.from_xyxy(39, 51, 156, 179)]
    print(record.detection.masks[0])
    assert record.detection.masks[0].to_erles(None, None) == EncodedRLEs(
        [
            {
                "size": [297, 300],
                "counts": b"fQ9:l86L3L5aHAk5f0nI@T1;n19jL_OS1>o1c1jMdNR2^1jMfNT2\\1dMmNY2V1bMoN\\2R1aMRO]2o0aMSO]2o0aMTO]2n0_MUO`2l0^MVO`2m0]MUOb2o0XMTOg2P1TMROk2g300O1O100O10000O2N2O1O3M:Fe0[O2NO1O1O1O1O1O1O2N1O1O2N1O1O1O2N1O2N1O00000000000000000000000000000000000000000000000000000000000000000000O100O100O1O1O1O100O1O1O1O1O1O1O100O1O1O1O1O1O1O100O1O100O10000O1000oJbNW2]1^MoNb2Q1oL_OP3b0jLCV3=gLFY3:fLHY39dLJ[37aLL_35^LNa34ZLOf34PL4o3OjK5V4`2O1O1O001O1O1O001O00001O1O001O1O1O1O001O1O001O1O001O001O00001O1O00001O1O001O1O001O001O00001O1O1O1O2N2N3M2N3M2N3M4L>B8H3M4L3M4L1OO1O1O1O1O1O1M3O00000O1O10000001N1N200O2O0N201N1N2O2O1N10000N2O1O2N100N3N2M2N3N2O1O1O1N2O2N1O1N3N1N3N2M3L6IfU6",
            }
        ]
    )



__all__ = ["NUM_CLASSES", "load_data"]

from icevision.imports import *
from icevision.core import *
from icevision.utils import *

NUM_CLASSES = 1 + 1


def load_data(force_download: bool = False):
    url = "https://s3.amazonaws.com/fast-ai-sample/biwi_sample.tgz"
    save_dir = get_data_dir() / "biwi"
    save_dir.mkdir(exist_ok=True)
    tar_file = save_dir / "biwi_sample.tgz"

    if not tar_file.exists() or force_download:
        download_and_extract(url=url, save_path=tar_file)

    return save_dir



__all__ = ["faster_rcnn_resnet50_fpn"]

from icevision.all import *
from icedata.utils import load_model_weights_from_url
from icedata.datasets.coco.data import NUM_CLASSES


def faster_rcnn_resnet50_fpn() -> nn.Module:
    weights_url = (
        "https://download.pytorch.org/models/fasterrcnn_resnet50_fpn_coco-258fb6c6.pth"
    )
    model = models.torchvision.faster_rcnn.model(num_classes=NUM_CLASSES)
    load_model_weights_from_url(model=model, url=weights_url)
    return model



__all__ = ["load_data"]

from icevision.imports import *


def load_data():
    return logger.info(
        """
    MANUALLY download AND unzip the dataset from https://cg.cs.tsinghua.edu.cn/dataset/form.html?dataset=ochuman. 
    You will need the path to the `ochuman.json` annotations file and the `images` directory.
    """
    )



__all__ = ["NUM_CLASSES", "class_map", "load_data", "load"]

from icevision.imports import *
from icevision.core import *
from icevision.utils import *


_CLASSES = sorted(
    {
        "Abyssinian",
        "great_pyrenees",
        "Bombay",
        "Persian",
        "samoyed",
        "Maine_Coon",
        "havanese",
        "beagle",
        "yorkshire_terrier",
        "pomeranian",
        "scottish_terrier",
        "saint_bernard",
        "Siamese",
        "chihuahua",
        "Birman",
        "american_pit_bull_terrier",
        "miniature_pinscher",
        "japanese_chin",
        "British_Shorthair",
        "Bengal",
        "Russian_Blue",
        "newfoundland",
        "wheaten_terrier",
        "Ragdoll",
        "leonberger",
        "english_cocker_spaniel",
        "english_setter",
        "staffordshire_bull_terrier",
        "german_shorthaired",
        "Egyptian_Mau",
        "boxer",
        "shiba_inu",
        "keeshond",
        "pug",
        "american_bulldog",
        "basset_hound",
        "Sphynx",
    }
)
NUM_CLASSES = len(_CLASSES) + 1


def class_map(background: int = "background"):
    return ClassMap(classes=_CLASSES, background=background)


def load_data(force_download: bool = False):
    base_url = "http://www.robots.ox.ac.uk/~vgg/data/pets/data"
    save_dir = get_data_dir() / "pets"
    save_dir.mkdir(exist_ok=True)

    url_images = os.path.join(base_url, "images.tar.gz")
    images_tar_file = save_dir / "images.tar.gz"
    if not images_tar_file.exists() or force_download:
        download_and_extract(url=url_images, save_path=images_tar_file)

    annotations_tar_file = save_dir / "annotations.tar.gz"
    if not annotations_tar_file.exists() or force_download:
        url_annotations = os.path.join(base_url, "annotations.tar.gz")
        download_and_extract(url=url_annotations, save_path=annotations_tar_file)

    return save_dir


def load(force_download: bool = False):
    warnings.warn("load will be deprecated in 0.1.0, please use load_data instead")
    return load_data(force_download=force_download)



__all__ = ["generate_dataset"]

from icevision.imports import *
from icevision.utils import *


def generate_dataset(dataset_name: str):
    this_path = Path(__file__).absolute()
    templates_dir = this_path.parent.parent.parent / "templates/dataset"
    datasets_dir = this_path.parent.parent / "datasets"

    new_dataset_dir = datasets_dir / dataset_name
    try:
        print(f"\n- Trying to create {new_dataset_dir} folder\n")
        new_dataset_dir.mkdir()
        print(f"- {new_dataset_dir} folder created\n")
    except FileExistsError:
        raise FileExistsError(
            f"A dataset named {dataset_name} already exists, try a different name"
        )

    template_files = get_files(templates_dir, extensions=[".py", ".md"])
    for template_file in template_files:
        text = template_file.read_text()

        # do stuff with the file
        text = re.sub(r"TK_DATASET_NAME", dataset_name, text)

        new_filepath = (
            datasets_dir / dataset_name / template_file.relative_to(templates_dir)
        )
        new_filepath.parent.mkdir(exist_ok=True)
        with open(str(new_filepath), "w") as new_file:
            new_file.write(text)
            print(f"   - {new_filepath.relative_to(datasets_dir)} file created\n")

    # append to init
    icedata_dir = Path(__file__).resolve().parents[1]
    print("icedata_dir: ", icedata_dir)

    # Get the existing datasets list
    dir_to_search = icedata_dir / "datasets"
    ds_dirlist = sorted(
        [
            filename
            for filename in os.listdir(dir_to_search)
            if (
                os.path.isdir(os.path.join(dir_to_search, filename))
                and not filename.startswith("_")
            )
        ]
    )

    init_lines = []
    # Add existing datasets to list
    for ds_name in ds_dirlist:
        import_statement = f"from icedata.datasets import {ds_name}\n"
        init_lines.append(import_statement)

    # # Add new dataset to list
    # import_statement = f"from icedata.datasets import {dataset_name}\n"
    # init_lines.append(import_statement)

    # Update "__init__.py" file
    init_filepath = datasets_dir / "__init__.py"
    # init_lines = init_filepath.readlines()
    # init_lines.append(import_statement)

    with open(init_filepath, "w") as init_file:
        init_file.write("".join(init_lines))



import icedata.utils
import icedata.template

from icedata.utils import *
from icedata.datasets import *


try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__)



__all__ = ["load_model_weights_from_url", "load_data"]

from icevision.all import *


def load_model_weights_from_url(
    model: nn.Module, url: str, map_location=torch.device("cpu"), **kwargs
) -> None:
    state_dict = torch.hub.load_state_dict_from_url(
        url, map_location=map_location, **kwargs
    )
    model.load_state_dict(state_dict)


def load_data(
    url: Union[str, Path],
    name: Union[str, Path],
    force_download: bool = False,
    dest_dir: Union[str, Path] = None,
    gdrive: bool = False,
) -> Path:
    if not dest_dir:
        dest_dir = get_data_dir()

    dest_dir = Path(dest_dir) / name
    if not dest_dir.exists() or force_download:
        dest_dir.mkdir(exist_ok=True, parents=True)
        save_path = dest_dir / Path(url).name
        if gdrive:
            download_and_extract_gdrive(url=url, extract_dir=save_path.parent)
        else:
            download_and_extract(url=url, save_path=save_path)

    return dest_dir



__all__ = ["tf_efficientdet_lite0", "faster_rcnn_resnet50_fpn"]

from icevision.all import *
from icedata.utils import load_model_weights_from_url
from icedata.datasets.fridge.data import NUM_CLASSES


def _load_efficientdet(backbone, weights_url):
    model = models.ross.efficientdet.model(
        backbone=backbone(pretrained=False), num_classes=NUM_CLASSES, img_size=384
    )
    load_model_weights_from_url(model, weights_url)
    return model


def _load_faster_rcnn(backbone, weights_url):
    model = models.torchvision.faster_rcnn.model(
        num_classes=NUM_CLASSES, backbone=backbone
    )
    load_model_weights_from_url(model=model, url=weights_url)
    return model


def tf_efficientdet_lite0():
    weights_url = "https://github.com/airctic/model_zoo/releases/download/m2/fridge_tf_efficientdet_lite0.zip"
    return _load_efficientdet(models.ross.efficientdet.backbones.tf_lite0, weights_url)


def faster_rcnn_resnet50_fpn():
    weights_url = "https://github.com/airctic/model_zoo/releases/download/m1/fridge_faster_resnetfpn50.zip"
    return _load_faster_rcnn(None, weights_url)



__all__ = ["NUM_CLASSES", "load_data", "class_map"]

from icevision.imports import *
from icevision.utils import *
from icevision.core import *

NUM_CLASSES = 200 + 1


# TODO: Need to install gdown
def load_data(force_download: bool = False) -> Path:
    images_url = "https://drive.google.com/uc?id=1GDr1OkoXdhaXWGA8S3MAq3a522Tak-nx&export=download"
    lists_url = "https://drive.google.com/uc?export=download&id=1vZuZPqha0JjmwkdaS_XtYryE3Jf5Q1AC"
    annotations_url = "https://drive.google.com/uc?export=download&id=16NsbTpMs5L6hT4hUJAmpW2u7wH326WTR"
    data_dir = get_data_dir() / "birds"
    data_dir.mkdir(exist_ok=True, parents=True)

    if not len(data_dir.ls()) == 0 or force_download:
        download_and_extract_gdrive(
            url=images_url, filename="images.tgz", extract_dir=data_dir
        )
        download_and_extract_gdrive(
            url=lists_url, filename="lists.tgz", extract_dir=data_dir
        )
        download_and_extract_gdrive(
            url=annotations_url, filename="annotations.tgz", extract_dir=data_dir
        )

    return data_dir


def class_map(data_dir: Union[str, Path], background: Optional[int] = 0) -> ClassMap:
    classes_filepath = Path(data_dir) / "lists/classes.txt"
    classes = classes_filepath.read_text().splitlines()
    return ClassMap(classes, background=background)



__all__ = ["dataset"]

from icevision.all import *
from .parser import *


def dataset(
    data_dir: Path,
    size: int = 384,
    presize: int = 512,
    data_splitter: Optional[DataSplitter] = None,
):
    _parser = parser(data_dir=data_dir)

    train_records, valid_records = _parser.parse(data_splitter=data_splitter)

    train_tfms = tfms.A.Adapter(
        [*tfms.A.aug_tfms(size=size, presize=presize), tfms.A.Normalize()]
    )
    valid_tfms = tfms.A.Adapter([*tfms.A.resize_and_pad(size), tfms.A.Normalize()])

    train_ds = Dataset(train_records, train_tfms)
    valid_ds = Dataset(valid_records, valid_tfms)

    return train_ds, valid_ds



__all__ = ["parser_bbox", "parser_mask", "parser_keypoints"]

from icevision.imports import *
from icevision.core import *
from icevision import parsers


parser_bbox = parsers.COCOBBoxParser
parser_mask = parsers.COCOMaskParser
parser_keypoints = parsers.COCOKeyPointsParser



__all__ = ["parser"]

from icevision.all import *
from icevision.core.record_defaults import KeypointsRecord


def parser(data_dir: Path):
    annotations_filepath = data_dir / "biwi_sample/centers.pkl"
    img_dir = data_dir / "biwi_sample/images/"

    return BIWIParser(annotations_filepath=annotations_filepath, img_dir=img_dir)


class BIWIKeypointsMetadata(KeypointsMetadata):
    labels = ["nose"]


class BIWIParser(Parser):
    def __init__(self, annotations_filepath, img_dir, idmap=None):
        super().__init__(template_record=self.template_record(), idmap=idmap)
        self.annotations_dict = pickle.load(open(Path(annotations_filepath), "rb"))
        self.img_dir = Path(img_dir)
        self.class_map = ClassMap(["nose"])

    def __iter__(self):
        yield from self.annotations_dict.items()

    def __len__(self):
        return len(self.annotations_dict)

    def template_record(self) -> BaseRecord:
        return KeypointsRecord()

    def record_id(self, o):
        return int(o[0].replace(".jpg", ""))

    def filepath(self, o):
        return self.img_dir / o[0]

    def keypoints(self, o):
        y, x = o[1].tolist()
        return [KeyPoints.from_xyv([x, y, 1], BIWIKeypointsMetadata)]

    def image_width_height(self, o) -> Tuple[int, int]:
        return get_img_size(self.filepath(o))

    def labels(self, o) -> List[Hashable]:
        return [1]

    def bboxes(self, o) -> List[BBox]:
        w, h = get_img_size(self.filepath(o))
        return [BBox.from_xywh(0, 0, w, h)]

    def parse_fields(self, o, record, is_new):
        if is_new:
            record.set_filepath(self.filepath(o))
            record.set_img_size(self.image_width_height(o))

        record.detection.set_class_map(self.class_map)
        record.detection.add_labels_by_id(self.labels(o))
        record.detection.add_bboxes(self.bboxes(o))
        record.detection.add_keypoints(self.keypoints(o))



__all__ = ["parser", "PetsBBoxParser", "PetsMaskParser", "PetsMaskFile"]

from icevision.imports import *
from icevision.core import *
from icevision import parsers
from .data import *


def parser(data_dir: Path, mask=False):
    annotations_dir = data_dir / "annotations/xmls"
    images_dir = data_dir / "images"
    masks_dir = data_dir / "annotations/trimaps"

    if not mask:
        parser = PetsBBoxParser(
            annotations_dir=annotations_dir,
            images_dir=images_dir,
            class_map=class_map(),
        )

    else:
        parser = PetsMaskParser(
            annotations_dir=annotations_dir,
            images_dir=images_dir,
            masks_dir=masks_dir,
            class_map=class_map(),
        )

    return parser


class PetsBBoxParser(parsers.VOCBBoxParser):
    def labels(self, o) -> List[Hashable]:
        name = re.findall(r"^(.*)_\d+$", o.stem)[0]

        # there is an image with two cats (same breed)
        num_objs = len(self._root.findall("object"))

        return [name] * num_objs


class PetsMaskParser(parsers.VOCMaskParser, PetsBBoxParser):
    def masks(self, o) -> List[Mask]:
        mask_file = self._record_id2maskfile[self.record_id(o)]
        return [PetsMaskFile(mask_file)]


class PetsMaskFile(VocMaskFile):
    """Extension of `MaskFile` for Pets masks.
    Invert 0s and 1s in the mask (the background is orignally 1 in the pets masks)
    Removes the color pallete and optionally drops void pixels.

    Args:
        drop_void (bool): drops the void pixels, which should have the value 255.
    """

    def to_mask(self, h, w) -> MaskArray:
        mask = super().to_mask(h=h, w=w)
        mask.data = 1 - mask.data
        return mask



from icevision.imports import Path
import icedata


def test_load_data(tmpdir):
    url = "https://github.com/airctic/icedata/archive/0.0.2post1.tar.gz"

    data_dir = icedata.load_data(url, "test_data")

    assert data_dir.exists()
    assert (data_dir / "0.0.2post1.tar.gz").exists()



__all__ = ["parser", "BirdsParser", "BirdMaskFile"]

from icevision.imports import *
from icevision.utils import *
from icevision.core import *
from icevision.parsers import Parser
from icedata.datasets.birds.data import class_map as class_map_fn


def parser(data_dir: Union[str, Path], class_map: Optional[ClassMap] = None) -> Parser:
    return BirdsParser(data_dir=data_dir, class_map=class_map)


class BirdsParser(Parser):
    def __init__(self, data_dir, class_map: Optional[ClassMap] = None):
        self.class_map = class_map or class_map_fn(data_dir)

        self.mat_filepaths = get_files(
            data_dir / "annotations-mat", extensions=[".mat"]
        )

        super().__init__(template_record=self.template_record())

    def __iter__(self) -> Any:
        yield from self.mat_filepaths

    def __len__(self) -> int:
        return len(self.mat_filepaths)

    def template_record(self) -> BaseRecord:
        return InstanceSegmentationRecord()

    def record_id(self, o) -> Hashable:
        return o.stem

    def parse_fields(self, o, record: BaseRecord, is_new: bool) -> None:
        if is_new:
            record.set_filepath(self.filepath(o))
            record.set_img_size(self.image_width_height(o))
            record.detection.set_class_map(self.class_map)

        record.detection.add_labels(self.labels(o))
        record.detection.add_bboxes(self.bboxes(o))
        record.detection.add_masks(self.masks(o))

    def filepath(self, o) -> Union[str, Path]:
        parts = list(o.parts)
        parts[-3] = "images"
        return Path(*parts).with_suffix(".jpg")

    def image_width_height(self, o) -> Tuple[int, int]:
        return get_img_size(self.filepath(o))

    def masks(self, o) -> List[Mask]:
        return [BirdMaskFile(o)]

    def bboxes(self, o) -> List[BBox]:
        import scipy.io

        mat = scipy.io.loadmat(str(o))
        bbox = mat["bbox"]
        xyxy = [int(bbox[pos]) for pos in ["left", "top", "right", "bottom"]]
        return [BBox.from_xyxy(*xyxy)]

    def labels(self, o) -> List[Hashable]:
        return [o.parent.name]
        # class_name = o.parent.name
        # return [self.class_map.get_name(class_name)]


class BirdMaskFile(MaskFile):
    def to_mask(self, h, w):
        import scipy.io

        mat = scipy.io.loadmat(str(self.filepath))
        return MaskArray(mat["seg"])[None]



__all__ = ["parser"]

from icevision.all import *


def parser(data_dir: Path):
    parser = parsers.VOCBBoxParser(
        annotations_dir=data_dir / "odFridgeObjects/annotations",
        images_dir=data_dir / "odFridgeObjects/images",
        class_map=ClassMap(
            ["milk_bottle", "carton", "can", "water_bottle"],
        ),
    )

    return parser



import icedata
from icevision.all import *


def test_parser(data_dir):
    class_map = icedata.fridge.class_map()
    parser = icedata.fridge.parser(data_dir)

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 5
    record = records[0]

    assert record.filepath.name == "10.jpg"
    assert int(record.record_id) == 10
    assert record.height == 666
    assert record.width == 499
    assert record.detection.labels == ["carton", "milk_bottle"]

    expected = [BBox.from_xyxy(86, 102, 216, 439), BBox.from_xyxy(150, 377, 445, 490)]
    assert record.detection.bboxes == expected



__all__ = ["mask_rcnn_resnet50_fpn"]

from warnings import warn

from icevision.all import *
from icedata.utils import load_model_weights_from_url
from icedata.datasets.pennfudan.data import NUM_CLASSES


def mask_rcnn_resnet50_fpn() -> nn.Module:
    warn(
        "Weights can currently not be loaded from state dict. This issue will be fixed in the future."
    )
    # weights_url = "https://github.com/airctic/icedata/releases/download/m1/pennfudan_maskrcnn_resnet50_fpn.zip"
    # weights_url = "https://github.com/airctic/model_zoo/releases/download/pennfudan_maskrcnn_resnet50fpn/pennfudan_maskrcnn_resnet50fpn.zip"
    model = models.torchvision.faster_rcnn.model(num_classes=NUM_CLASSES)
    # load_model_weights_from_url(model=model, url=weights_url)
    return model



import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.ochuman.parser(
        data_dir / "annotations/ochuman.json", data_dir / "images"
    )

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 10
    record = records[3]

    assert record.filepath.name == "000004.jpg"
    assert record.record_id == 4
    assert record.detection.labels == ["person", "person"]
    assert record.height == 333
    assert record.width == 500



import icedata


def test_class_map():
    num_classes = icedata.fridge.NUM_CLASSES
    class_map = icedata.fridge.class_map()

    assert num_classes == len(class_map) == 5
    assert class_map.get_by_id(0) == "background"
    assert class_map.get_by_id(-1) == "water_bottle"



__all__ = ["class_map", "load_data"]

from icevision.imports import *
from icevision.core import *
from icevision.utils import *

_CLASSES = sorted(
    {
        "person",
        "bird",
        "cat",
        "cow",
        "dog",
        "horse",
        "sheep",
        "aeroplane",
        "bicycle",
        "boat",
        "bus",
        "car",
        "motorbike",
        "train",
        "bottle",
        "chair",
        "diningtable",
        "pottedplant",
        "sofa",
        "tvmonitor",
    }
)


def class_map(background: int = 0):
    return ClassMap(classes=_CLASSES, background=background)


def load_data(force_download=False):
    url = "http://host.robots.ox.ac.uk/pascal/VOC/voc2012/VOCtrainval_11-May-2012.tar"

    save_dir = get_data_dir() / "voc"
    save_dir.mkdir(exist_ok=True)
    tar_file = save_dir / "trainval.tar"

    if not tar_file.exists() or force_download:
        download_url(url=url, save_path=tar_file)
        # extract file
        shutil.unpack_archive(str(tar_file), str(save_dir))
        # move extract files so they are placed at save_dir
        files_dir = save_dir / "VOCdevkit/VOC2012"
        for file in files_dir.ls():
            shutil.move(str(file), str(save_dir))
        shutil.rmtree(str(files_dir.parent))

    return save_dir



__all__ = ["dataset"]

from icevision.all import *
from .parser import *


def dataset(
    data_dir: Path,
    size: int = 384,
    presize: int = 512,
    data_splitter: Optional[DataSplitter] = None,
    task: str = "bbox",
):
    if task == "bbbox":
        parser = parsers.COCOBBoxParser
    elif task == "mask":
        parser = parsers.COCOMaskParser
    elif task == "keypoints":
        parser = parsers.COCOKeyPointsParser
    else:
        raise ValueError(
            f"The 'task' argmument passed is '{task}'. 'task' argument must be one o these 3 values 'bbox', 'mask, or 'keypoints'. "
        )

    _parser = parser(
        annotations_file=data_dir / "annotations.json", img_dir=data_dir / "images"
    )

    train_records, valid_records = _parser.parse(data_splitter=data_splitter)

    train_tfms = tfms.A.Adapter(
        [*tfms.A.aug_tfms(size=size, presize=presize), tfms.A.Normalize()]
    )
    valid_tfms = tfms.A.Adapter([*tfms.A.resize_and_pad(size), tfms.A.Normalize()])

    train_ds = Dataset(train_records, train_tfms)
    valid_ds = Dataset(valid_records, valid_tfms)

    return train_ds, valid_ds



__all__ = ["plot_size_histogram"]

from icevision.all import *


def plot_size_histogram(records: List[RecordType], show=True) -> List[plt.Axes]:
    heights = []
    widths = []
    for record in records:
        heights.append(record["height"])
        widths.append(record["width"])

    fig, axs = plt.subplots(ncols=2)

    def plot_hist(ax, values, title):
        ax.hist(values, bins=30)
        ax.set_title(title)

    plot_hist(axs[0], heights, "heights")
    plot_hist(axs[1], widths, "widths")

    if show:
        plt.show()

    return axs



__all__ = ["NUM_CLASSES", "class_map", "load_data"]

from icevision.imports import *
from icevision.utils import *
from icevision.core import *

_CLASSES = sorted({"milk_bottle", "carton", "can", "water_bottle"})
NUM_CLASSES = 4 + 1


def class_map(background: Optional[int] = "background") -> ClassMap:
    return ClassMap(classes=_CLASSES, background=background)


def load_data(force_download=False):
    base_url = "https://cvbp-secondary.z19.web.core.windows.net/datasets/object_detection/odFridgeObjects.zip"
    save_dir = get_data_dir() / "fridge"
    save_dir.mkdir(exist_ok=True)

    save_path = save_dir / "data.zip"
    if not save_path.exists() or force_download:
        download_and_extract(url=base_url, save_path=save_path)

    return save_dir



import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.pennfudan.parser(data_dir)

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 5
    record = records[0]

    assert record.filepath.name == "FudanPed00001.png"
    assert record.record_id == 1
    assert record.detection.labels == ["person", "person"]
    assert record.height == 536
    assert record.width == 559

    expected = [BBox.from_xyxy(160, 182, 302, 431), BBox.from_xyxy(420, 171, 535, 486)]
    assert record.detection.bboxes == expected
